﻿
Imports System.Windows.Threading

Public Class ChatForm

    Private Shared m_objChatForm As ChatForm = Nothing
    Private m_objContactNameList As List(Of String) = New List(Of String)()

    Private m_objVaxUserAgent As cVaxSIPUserAgentExCOM = Nothing
    Private m_bTyping As Boolean = False
    Private mTimerMsgTyping As DispatcherTimer = Nothing

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Public Sub New()

        m_objChatForm = Me
        m_objVaxUserAgent = cVaxSIPUserAgentExCOM.GetOBJ()

        InitializeComponent()

        mTimerMsgTyping = New DispatcherTimer()
        AddHandler mTimerMsgTyping.Tick, AddressOf OnTypingTimer
        mTimerMsgTyping.Interval = New TimeSpan(0, 0, 2)

        LabelChatTyping.Content = ""

    End Sub

    Public Shared Function GetOBJ() As ChatForm
        Return m_objChatForm
    End Function

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Public Sub LoadContactAll()

        m_objContactNameList.Clear()
        ListBoxContacts.Items.Clear()

        Dim objList As ArrayList = New ArrayList()
        cStoreChat.GetContactAll(objList)

        For nCount As Integer = 0 To objList.Count - 1

            Dim sContactName As String = CType(objList(nCount), String)
            ListBoxContacts.Items.Add(sContactName)

            m_objContactNameList.Add(sContactName)
            m_objVaxUserAgent.ChatAddContact(sContactName)

        Next

        If ListBoxContacts.SelectedIndex = -1 AndAlso ListBoxContacts.Items.Count <> 0 Then
            ListBoxContacts.SelectedIndex = 0
            LoadContactMsgAll(CType(objList(0), String))
        End If

    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Public Sub OnVaxOnline()

        LabelChatTyping.Content = ""
        m_bTyping = False

        LoadContactAll()
        UpdateStatusCombo()

    End Sub

    Public Sub OnVaxOffline()

        mTimerMsgTyping.Stop()
        ListBoxStatusLog.Items.Clear()

        m_objContactNameList.Clear()
        ListBoxContacts.Items.Clear()

    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Private Sub UpdateStatusCombo()

        ComboMyChatStatus.Items.Clear()

        ComboMyChatStatus.Items.Add("Online")
        ComboMyChatStatus.Items.Add("Offline")
        ComboMyChatStatus.Items.Add("Away")
        ComboMyChatStatus.Items.Add("On the phone")
        ComboMyChatStatus.Items.Add("Busy")

        ComboMyChatStatus.SelectedIndex = cVaxSIPUserAgentCOM.VAX_CONTACT_STATUS_OFFLINE

    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Private Sub BtnAddContact_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)

        If EditContactName.Text.Length = 0 Then
            Return
        End If

        If Not m_objVaxUserAgent.ChatAddContact(EditContactName.Text) Then
            Return
        End If

        ListBoxContacts.Items.Add(EditContactName.Text)
        m_objContactNameList.Add(EditContactName.Text)

        cStoreChat.AddContact(EditContactName.Text)

        If ListBoxContacts.SelectedIndex = -1 AndAlso ListBoxContacts.Items.Count <> 0 Then
            ListBoxContacts.SelectedIndex = 0
        End If

        EditContactName.Text = ""

    End Sub

    Private Sub BtnRemoveContact_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)

        Dim nSelectedIndex As Integer = ListBoxContacts.SelectedIndex
        If nSelectedIndex = -1 Then Return

        Dim Result As MessageBoxResult = MessageBox.Show("Do you really want to remove this Contact?", "Caution", MessageBoxButton.YesNo, MessageBoxImage.Question)
        If Result = MessageBoxResult.No Then Return

        Dim sContactName As String = m_objContactNameList(nSelectedIndex)

        m_objVaxUserAgent.ChatRemoveContact(sContactName)
        ListBoxContacts.Items.RemoveAt(nSelectedIndex)
        m_objContactNameList.RemoveAt(nSelectedIndex)

        cStoreChat.RemoveContact(sContactName)

        If ListBoxContacts.SelectedIndex = -1 AndAlso ListBoxContacts.Items.Count <> 0 Then
            ListBoxContacts.SelectedIndex = 0
        End If

    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Private Sub ListBoxContacts_SelectionChanged(ByVal sender As System.Object, ByVal e As System.Windows.Controls.SelectionChangedEventArgs)

        Dim nSelectedIndex As Integer = ListBoxContacts.SelectedIndex
        If nSelectedIndex = -1 Then Return

        ListBoxMessages.Items.Clear()
        LoadContactMsgAll(m_objContactNameList(nSelectedIndex))

    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Public Sub LoadContactMsgAll(ByVal sContactName As String)

        ListBoxMessages.Items.Clear()

        Dim objList As ArrayList = New ArrayList()
        cStoreChat.GetMsgAll(sContactName, objList)

        For nCount As Integer = 0 To objList.Count - 1
            Dim sMsg As String = CType(objList(nCount), String)
            ListBoxMessages.Items.Add(sMsg)
        Next

    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Private Sub ComboMyChatStatus_SelectionChanged(ByVal sender As System.Object, ByVal e As System.Windows.Controls.SelectionChangedEventArgs)

        Dim nSelectedIndex As Integer = ComboMyChatStatus.SelectedIndex
        If nSelectedIndex = -1 Then Return

        m_objVaxUserAgent.ChatSetMyStatus(nSelectedIndex)

    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Private Sub BtnSendMessage_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)

        If ListBoxContacts.SelectedIndex = -1 Then Return
        If EditSendMessage.Text.Length = 0 Then Return

        Dim sContactName As String = m_objContactNameList(ListBoxContacts.SelectedIndex)

        If Not m_objVaxUserAgent.ChatSendMessageText(sContactName, EditSendMessage.Text) Then
            Return
        End If

        Dim sMsg As String = "Me: " & EditSendMessage.Text
        ListBoxMessages.Items.Add(sMsg)

        cStoreChat.AddMsg(sContactName, sMsg)
        EditSendMessage.Text = ""

    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Public Sub OnChatSendMsgTextSuccess(ByVal sUserName As String, ByVal sMsgText As String, ByVal nUserValue As Long)
        AddToStatusLog("Send Message to " & sUserName & " Success.")
    End Sub

    Public Sub OnChatSendMsgTextFailed(ByVal sUserName As String, ByVal nStatusCode As Integer, ByVal sReasonPhrase As String, ByVal sMsgText As String, ByVal nUserValue As Long)
        AddToStatusLog("Send Message to " & sUserName & " Failed.")
    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Private Sub BtnClearMessage_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)

        If ListBoxContacts.SelectedIndex = -1 Then Return
        ListBoxMessages.Items.Clear()

        Dim sContactName As String = m_objContactNameList(ListBoxContacts.SelectedIndex)
        cStoreChat.RemoveMsgAll(sContactName)

    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Private Sub AddToStatusLog(ByVal sLog As String)
        ListBoxStatusLog.Items.Add(sLog)
    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Private Function GetStatusText(ByVal nStatusId As Integer) As String

        Dim sStatus As String = ""

        Select Case nStatusId
            Case cVaxSIPUserAgentCOM.VAX_CONTACT_STATUS_ONLINE
                sStatus = "Online"
            Case cVaxSIPUserAgentCOM.VAX_CONTACT_STATUS_OFFLINE
                sStatus = "Offline"
            Case cVaxSIPUserAgentCOM.VAX_CONTACT_STATUS_AWAY
                sStatus = "Away"
            Case cVaxSIPUserAgentCOM.VAX_CONTACT_STATUS_ON_PHONE
                sStatus = "On the phone"
            Case cVaxSIPUserAgentCOM.VAX_CONTACT_STATUS_BUSY
                sStatus = "Busy"
            Case cVaxSIPUserAgentCOM.VAX_CONTACT_STATUS_UNKNOWN
                sStatus = "Offline"
        End Select

        Return sStatus

    End Function

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Public Sub OnChatSubscribeSuccess(ByVal sUserName As String)
        AddToStatusLog("Chat Subscribe " & sUserName & " Success.")
    End Sub

    Public Sub OnChatSubscribeFailed(ByVal sUserName As String, ByVal nStatusCode As Integer, ByVal sReasonPhrase As String)
        AddToStatusLog("Chat Subscribe " & sUserName & " Failed.")
    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Public Sub OnChatContactStatus(ByVal sUserName As String, ByVal nStatusId As Integer)

        Dim nSelectedIndex As Integer = ListBoxContacts.SelectedIndex

        For nContactCount As Integer = 0 To m_objContactNameList.Count - 1

            Dim sContactName As String = m_objContactNameList(nContactCount)
            sContactName = sContactName.ToUpper()

            If sContactName = sUserName.ToUpper() Then
                Dim sStatus As String = sUserName & " [" & GetStatusText(nStatusId) & "]"
                ListBoxContacts.Items(nContactCount) = sStatus
                Exit For
            End If

        Next

        ListBoxContacts.SelectedIndex = nSelectedIndex

    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Public Sub OnChatRecvMsgText(ByVal sUserName As String, ByVal sMsgText As String, ByVal bIsContact As Boolean)

        Dim nStart As Integer = sMsgText.IndexOf(">")
        Dim sMessage As String = ""

        If nStart = -1 Then
            sMessage = sUserName & ": " & sMsgText
        Else
            nStart = nStart + 1
            Dim nEnd As Integer = sMsgText.IndexOf("<", nStart)
            sMessage = sUserName & ": " & sMsgText.Substring(nStart, nEnd - nStart)
        End If

        ListBoxMessages.Items.Add(sMessage)
        cStoreChat.AddMsg(sUserName, sMessage)

    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Public Sub OnChatRecvMsgTypingStart(ByVal sUserName As String)

        m_bTyping = True
        LabelChatTyping.Content = sUserName & ": Typing"

        mTimerMsgTyping.Stop()
        mTimerMsgTyping.Start()

        AdjustTypingState()

    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Private Sub OnTypingTimer(ByVal sender As System.Object, ByVal e As System.EventArgs)
        AdjustTypingState()
    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Private Sub AdjustTypingState()

        If Not m_bTyping Then
            mTimerMsgTyping.[Stop]()
            OnChatRecvMsgTypingStop()
        Else
            m_bTyping = False
        End If

    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Public Sub OnChatRecvMsgTypingStop()
        LabelChatTyping.Content = ""
    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Private Sub EditSendMessage_KeyDown(ByVal sender As Object, ByVal e As KeyEventArgs)

        If ListBoxContacts.SelectedIndex = -1 Then Return

        Dim sContactName As String = m_objContactNameList(ListBoxContacts.SelectedIndex)
        m_objVaxUserAgent.ChatSendMessageTyping(sContactName)

    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Public Sub OnChatSendMsgTypingSuccess(ByVal sUserName As String, ByVal nUserValue As Long)
        AddToStatusLog("Send Typing to " & sUserName & " Success.")
    End Sub

    Public Sub OnChatSendMsgTypingFailed(ByVal sUserName As String, ByVal nStatusCode As Integer, ByVal sReasonPhrase As String, ByVal nUserValue As Long)
        AddToStatusLog("Send Typing to " & sUserName & " Failed.")
    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Private Sub BtnClearStatusLog_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)
        ListBoxStatusLog.Items.Clear()
    End Sub

    '////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////

    Private Sub ScrollViewer_OnListBoxScrollChanged(ByVal sender As System.Object, ByVal e As System.Windows.Controls.ScrollChangedEventArgs)

        If TypeOf e.OriginalSource Is ScrollViewer AndAlso Math.Abs(e.ExtentHeightChange) > 0.0 Then
            Dim hScroll As ScrollViewer = CType(e.OriginalSource, ScrollViewer)
            hScroll.ScrollToBottom()
        End If

    End Sub

End Class

