﻿Imports System
Imports System.Collections
Imports Microsoft.Win32

Class cStoreRegistry

    Const REGISTRY_ROOT_PATH As String = "SOFTWARE\VaxPhoneSDKv6"
    Private m_hKey As RegistryKey

    Public Sub New()
        Open()
    End Sub

    Private Sub Open()
        If m_hKey IsNot Nothing Then Return
        m_hKey = Registry.CurrentUser.CreateSubKey(REGISTRY_ROOT_PATH)
    End Sub

    Public Sub Close()
        If m_hKey IsNot Nothing Then
            m_hKey.Close()
            m_hKey = Nothing
        End If
    End Sub

    Private Function CreateKey(ByVal sKeyName As String) As RegistryKey

        If m_hKey Is Nothing Then
            Return Nothing
        End If

        If sKeyName.Length = 0 Then
            Return Nothing
        End If

        Return m_hKey.CreateSubKey(sKeyName)

    End Function

    Private Function OpenKey(ByVal sKeyName As String, ByVal bWritable As Boolean) As RegistryKey

        If m_hKey Is Nothing Then
            Return Nothing
        End If

        If sKeyName.Length = 0 Then
            Return Nothing
        End If

        Return m_hKey.OpenSubKey(sKeyName, bWritable)

    End Function

    Public Function SetTEXT(ByVal sKeyName As String, ByVal sValueName As String, ByVal sValueData As String) As Boolean

        If sValueName.Length = 0 Then
            Return False
        End If


        Dim hKey As RegistryKey = CreateKey(sKeyName)
        If hKey Is Nothing Then Return False

        hKey.SetValue(sValueName, sValueData, RegistryValueKind.String)
        hKey.Close()

        Return True

    End Function

    Public Function SetINT(ByVal sKeyName As String, ByVal sValueName As String, ByVal nValueData As Integer) As Boolean

        If sValueName.Length = 0 Then Return False

        Dim hKey As RegistryKey = CreateKey(sKeyName)
        If hKey Is Nothing Then Return False

        hKey.SetValue(sValueName, nValueData, RegistryValueKind.DWord)
        hKey.Close()

        Return True

    End Function

    Public Function SetBOOL(ByVal sKeyName As String, ByVal sValueName As String, ByVal bValueData As Boolean) As Boolean
        Dim nValueData As Integer = If(bValueData, 1, 0)
        Return Me.SetINT(sKeyName, sValueName, nValueData)
    End Function

    Public Sub GetTEXT(ByVal sKeyName As String, ByVal sValueName As String, ByRef sValueData As String, ByVal sValueDefault As String)

        If sValueName.Length = 0 Then
            sValueData = sValueDefault
            Return
        End If

        Dim hKey As RegistryKey = OpenKey(sKeyName, False)

        If hKey Is Nothing Then
            sValueData = sValueDefault
            Return
        End If

        sValueData = CStr(hKey.GetValue(sValueName, sValueDefault))
        hKey.Close()

    End Sub

    Public Function GetINT(ByVal sKeyName As String, ByVal sValueName As String, ByRef nValueData As Integer, ByVal nValueDefault As Integer) As Boolean

        If sValueName.Length = 0 Then
            nValueData = nValueDefault
            Return False
        End If

        Dim hKey As RegistryKey = OpenKey(sKeyName, False)

        If hKey Is Nothing Then
            nValueData = nValueDefault
            Return False
        End If

        nValueData = CInt(hKey.GetValue(sValueName, nValueDefault))
        hKey.Close()

        Return True

    End Function

    Public Sub GetBOOL(ByVal sKeyName As String, ByVal sValueName As String, ByRef bValueData As Boolean, ByVal bValueDefault As Boolean)

        Dim nValueData As Integer = 0
        Dim nValueDefault As Integer = If(bValueDefault, 1, 0)

        If Not GetINT(sKeyName, sValueName, nValueData, nValueDefault) Then
            bValueData = bValueDefault
            Return
        End If

        If nValueData = 0 Then
            bValueData = False
        Else
            bValueData = True
        End If

    End Sub

    Public Sub DeleteSubKey(ByVal sKeyName As String)
        If sKeyName.Length = 0 Then Return
        m_hKey.DeleteSubKeyTree(sKeyName, False)
    End Sub

    Public Function DeleteValue(ByVal sKeyRootName As String, ByVal sValueName As String) As Boolean

        If sValueName.Length = 0 Then
            Return False
        End If

        Dim hKey As RegistryKey = OpenKey(sKeyRootName, True)
        If hKey Is Nothing Then Return False

        hKey.DeleteValue(sValueName, False)
        hKey.Close()

        Return True

    End Function

    Public Function EnumValueNames(ByVal sKeyRootName As String, ByRef objList As ArrayList) As Boolean

        objList.Clear()

        Dim hKey As RegistryKey = OpenKey(sKeyRootName, False)
        If hKey Is Nothing Then Return False

        Dim arrayLogin As String() = CType(hKey.GetValueNames(), String())

        For nCount As Integer = 0 To arrayLogin.Length - 1
            objList.Add(arrayLogin(nCount))
        Next

        hKey.Close()
        Return True

    End Function

    Public Function EnumSubKeys(ByVal sKeyRootName As String, ByRef objList As ArrayList) As Boolean

        objList.Clear()

        Dim hKey As RegistryKey = OpenKey(sKeyRootName, False)
        If hKey Is Nothing Then Return False

        Dim arrayLogin As String() = CType(hKey.GetSubKeyNames(), String())

        For nCount As Integer = 0 To arrayLogin.Length - 1
            objList.Add(arrayLogin(nCount))
        Next

        hKey.Close()
        Return True

    End Function

    Public Function IsKeyExist(ByVal sKeyName As String) As Boolean

        Dim hKey As RegistryKey = OpenKey(sKeyName, False)
        If hKey Is Nothing Then Return False

        hKey.Close()
        Return True

    End Function

End Class
